% ConstructEgretShapeScript - Construct egret shape script template.
%
%   The shape script contains an ellipsoid, an angle, and two pairs of parallel bars.

clear
close all
addpath('common');

% lesson learned:
%  it is easier to watch the S1map, M1Map, S2Map, M2map when debugging.

%% specify an S3 template
% Firstly specify the absolute positions of ellispoids, then transform them
%   to positions on the subsampled lattice.

% load the S2 dictionary
load template_params
load result/transformedS2Template.mat

subsampleS2 = 3;
selectedRow = single([0 -45 -40 -20]);
selectedCol = single([0 -50 -30 -30]);
selectedTransform = single([114 7 18 18]-1);
selectedS2Ind = single([0 1 2 2]);
% selectedRow = 0;
% selectedCol = 0;
% selectedTransform = selectedTransform(2);
% selectedS2Ind = selectedS2Ind(2);
nF = length(selectedRow);
top = min(selectedRow);
bottom = max(selectedRow);
left = min(selectedCol);
right = max(selectedCol);
S3size = max(bottom-top,right-left);
selectedLambda = 3 * single(ones(1,nF));
selectedLogZ = single(4 * ones(1,nF));


%% display the S3 template
% TODO: When drawing the S3 template, re-center the features first. Do the same for S2 templates.  
margin = 60;
height = bottom - top + 1 + 2 * margin;
width = right - left + 1 + 2 * margin;
canvas = zeros(height,width);
for iF = 1:nF
    iT = selectedTransform(iF)+1;
    iS2 = selectedS2Ind(iF) + 1;
    patch = TransformedTemplate{iT,iS2}.sym;
    row = selectedRow(iF) + floor(height/2);
    col = selectedCol(iF) + floor(width/2);
    %
    % place the patch to (row,col)
    %
    % below: the bounding box inside the large canvas
    top2 = row - floor(size(patch,1)/2);
    left2 = col - floor(size(patch,2)/2);
    bottom2 = top2 + size(patch,1) - 1;
    right2 = left2 + size(patch,2) - 1;
    % 
    for c = 1:size(patch,2)
        for r = 1:size(patch,1)
            val = patch(r,c);
            if val > canvas(top2+r-1,left2+c-1)
                canvas(top2+r-1,left2+c-1) = val;
            end
        end
    end
end
canvas([1:2 end-1:end],:) = max(canvas(:))/2;
canvas(:,[1:2 end-1:end]) = max(canvas(:))/2;


category = 'egret';
h=figure; imshow(-canvas,[]);
mkdir(sprintf('result/%s',category));
saveas(h,sprintf('result/%s/template.png',category));
saveas(h,sprintf('result/%s/%s_template.eps',category,category), 'epsc');
close
S3Template = cell(1,1);
S3Template{1} = struct('selectedRow',floor(selectedRow/subsampleS2),'selectedCol',floor(selectedCol/subsampleS2),...
    'selectedInd',selectedS2Ind,'selectedTransform',selectedTransform,'selectedLambda',selectedLambda...
    ,'selectedLogZ',selectedLogZ);


%% Compute an adjacency matrix of S2 templates (used in computing M2 maps) 
nTransform = size(TransformedTemplate,1);
nS2 = size(TransformedTemplate,2);
templateAffMat = cell(nTransform,1);
for iT = 1:nTransform
    tmp = templateTransform{iT};
    s1 = tmp(1); rs1 = tmp(2); cs1 = tmp(3); r1 = tmp(4);
    neighbors = [];
    for jT = 1:nTransform
        tmp = templateTransform{jT};
        s2 = tmp(1); rs2 = tmp(2); cs2 = tmp(3); r2 = tmp(4);
        alpha1 = single(r1)/nGaborOri*pi ;
        alpha2 = single(r2)/nGaborOri*pi ;
        diff_alpha = alpha1 - alpha2;
        while diff_alpha < 0
            diff_alpha = diff_alpha + pi;
        end
        while diff_alpha >= pi
            diff_alpha = diff_alpha - pi;
        end
        if diff_alpha > pi/2
            diff_alpha = pi - diff_alpha;
        end
        if abs(s2 - s1) <= 0 ...
                && diff_alpha < pi/3;
            neighbors = [neighbors jT];
        end
    end
    templateAffMat{iT} = int32(neighbors-1);
end

% save
save(sprintf('result/%s/shapescript.mat',category),'templateAffMat','S3Template','nTransform','nS2','nF','S3size', 'subsampleS2');