% DisplayS3ScoreHistogram - Compare the distribution of S3 scores on natural images, vs.
%       its distribution on positive examples.
%   

clear
close all
addpath('common');

category = 'neg';

%% load the S2 (shape motif) dictionary and S3 (shape script) template
load template_params
load result/transformedS2Template.mat
load result/egret/shapescript.mat

%% load the testing image
%{
This can also be the comment
%}

Iname = {'neg04.jpg','neg05.jpg','neg06.jpg','neg07.jpg','neg08.jpg',...
    'neg21.jpg','neg22.jpg','neg23.jpg','neg24.jpg','neg25.jpg'};
imScale = [.5 .5 .5 .5 .5,...
    0.3 .5 .5 .5 .5];
binCenterS3 = [-100:20:400];

subsampleM1 = 1;
S3Hist = zeros(1,length(binCenterS3));
for iImage = 1:length(Iname)
    im = imread( sprintf('image/%s/%s',category,Iname{iImage}) );
    im = imresize( im,imScale(iImage),'nearest' );

    if size(im,3) == 3
        im = rgb2gray(im);
    end

    im = single(im);
    S1Map = applyfilterMultiScale({im},GaborFilter, nGaborOri);

    % sigmoid transform
    nImage = 1;
    heightS1Map = zeros(length(S1Map),1,'single');
    widthS1Map = zeros(length(S1Map),1,'single');
    for i = 1:length(S1Map)
        heightS1Map(i) = size(S1Map{i},1);
        widthS1Map(i) = size(S1Map{i},2);
    end
    tmpNGaborOri = 1;
    for i = 1:length(S1Map)
        Csigmoid(nImage, heightS1Map(i), widthS1Map(i), tmpNGaborOri, GaborSaturation, S1Map(i));
    end

    % compute M1 map
    sizeGaborPerScale = zeros(nGaborScale,1,'single');
    for iS = 1:nGaborScale
        sizeGaborPerScale(iS) = size( GaborFilter{(iS-1)*nGaborOri+1}, 1 );
    end
    locationShift = .2;
    oriShift = 1;
    [M1Map M1Trace M1RowShift M1ColShift M1OriShift] = ...
        CcomputeMAX1( nGaborOri, S1Map, locationShift, oriShift, sizeGaborPerScale, subsampleM1 );

    % compute S2 map
    S2Map = CcomputeSUM2( nGaborOri, M1Map, TransformedTemplate, subsampleS2 );

    % compute M2 map
    locationPerturbationFraction = 0.2; subsampleM2 = 1;
    M2Map = cell(nTransform, nS2);
    M2LocationTrace = cell(nTransform, nS2);
    M2TransformTrace = cell(nTransform, nS2);
    for iS2 = 1:nS2
        sizeTemplate = zeros(nTransform,1);
        for iT = 1:nTransform
            sizeTemplate(iT) = sqrt( ...
                size(TransformedTemplate{iT,iS2}.sym,1)...
                * size(TransformedTemplate{iT,iS2}.sym,2) ) / subsampleS2;
        end
        [M2Map(:,iS2) M2LocationTrace(:,iS2) M2TransformTrace(:,iS2) storedM2LocationShift] = ...
              CcomputeMAX2( templateAffMat, S2Map((iS2-1)*nTransform + (1:nTransform)), locationPerturbationFraction, ... 
                 int32(sizeTemplate), subsampleM2 );
        storedSizeS2Template{iS2} = sizeTemplate;
    end
    disp('done with computing M2');

    %% detection
    subsampleS3 = 1;
    %
    % Compute SUM 3 map :
    %
    S3Map = CcomputeSUM3_loglinear( M2Map, S3Template, subsampleS3, nTransform );
    for ii = 1:numel(S3Map)
        tmp = hist(S3Map{ii}(:),binCenterS3);
        S3Hist(:) = S3Hist(:) + tmp(:);
    end
    clear S2Map;
    clear M2Map;
    clear S3Map;
    clear M1Map;
    clear S1Map;
end



%% collect statistics on positive images

count = 1;

Iname = {'pelican 2_big.jpg','pelican3.jpg',...
    'pelican4.jpg','pelican5.jpg','pelican6.jpg',...
    'pelican7.jpg','pelican8.jpg','pelican9.jpg',...
    'pelican10.jpg'};
Iname_pelican = Iname;

category = 'pelican';
for i = 1:length(Iname)
    load(sprintf('result/%s/detectionResult%d_%s.mat',category,1,Iname{i}),...
                'im','mask','S3Map','M2Map');
    tmp = -1e10;
    for j = 1:numel(S3Map)
        tmp = max(tmp, max(S3Map{j}(:)));
    end
    S3Pos(count) = tmp;
    count = count + 1;
end

Iname = {'l1.jpg','l2.jpg','l3.jpg','l4.jpg','l5.jpg',...
    'l8.jpg','l10.jpg',...
    'l13.jpg','l14.jpg','l15.jpg',...
    'l17.jpg','l18.jpg','l19.jpg','l20.jpg',...
    'l21.jpg', 'l22.jpg',...
    'l26.jpg','l30.jpg',...
    'l4_dup.jpg'};
Iname_egret = Iname;
category = 'egret';
for i = 1:length(Iname)
    load(sprintf('result/%s/detectionResult%d_%s.mat',category,1,Iname{i}),...
                'im','mask','S3Map','M2Map');
    tmp = -1e10;
    for j = 1:numel(S3Map)
        tmp = max(tmp, max(S3Map{j}(:)));
    end
    S3Pos(count) = tmp;
    count = count + 1;
end

S3PosHist = hist(S3Pos,binCenterS3);

%% histogram
figure; plot(binCenterS3,S3Hist/sum(S3Hist(:)),'b--','LineWidth',3);
hold on;
plot(binCenterS3,S3PosHist/sum(S3PosHist(:)),'r-','LineWidth',3);

set(gca,'FontSize',12);
set(get(gca,'XLabel'),'String','S3 score','FontSize',18);
set(get(gca,'YLabel'),'String','Frequency','FontSize',18);
legend( {'natural','positive'},'FontSize',16,0);

saveas(gcf,'S3histogram.eps','epsc')
saveas(gcf,'S3histogram.png')