% FindPelican - A simple test case of detection using shape script.
%
%   Detect pelicans with a simple egret/pelican shape script template.
%
%   The shape script template is found at "result/egret/egret_template.eps" or
% "result/egret/template.png". The detection results together with S3 and S2 maps
% are found at "result/pelican".

clear
close all
addpath('common');

%% load the S2 (shape motif) dictionary and S3 (shape script) template
load template_params
load result/transformedS2Template.mat
load result/egret/shapescript.mat
category = 'pelican';

%% load the testing image
Iname = {'pelican 2_big.jpg','pelican3.jpg',...
    'pelican4.jpg','pelican5.jpg','pelican6.jpg',...
    'pelican7.jpg','pelican8.jpg','pelican9.jpg',...
    'pelican10.jpg'};
imScale = [0.25 0.4 0.4 0.4 ...
    0.22 0.36 0.5 0.4 0.3];

subsampleM1 = 1;
for iImage = 1:length(Iname)
    im = imread( sprintf('image/%s/%s',category,Iname{iImage}) );
    im = imresize( im,imScale(iImage),'nearest' );

    if size(im,3) == 3
        im = rgb2gray(im);
    end

    im = single(im);
    S1Map = applyfilterMultiScale({im},GaborFilter, nGaborOri);

    % sigmoid transform
    nImage = 1;
    heightS1Map = zeros(length(S1Map),1,'single');
    widthS1Map = zeros(length(S1Map),1,'single');
    for i = 1:length(S1Map)
        heightS1Map(i) = size(S1Map{i},1);
        widthS1Map(i) = size(S1Map{i},2);
    end
    tmpNGaborOri = 1;
    for i = 1:length(S1Map)
        Csigmoid(nImage, heightS1Map(i), widthS1Map(i), tmpNGaborOri, GaborSaturation, S1Map(i));
    end

    % compute M1 map
    sizeGaborPerScale = zeros(nGaborScale,1,'single');
    for iS = 1:nGaborScale
        sizeGaborPerScale(iS) = size( GaborFilter{(iS-1)*nGaborOri+1}, 1 );
    end
    locationShift = .2;
    oriShift = 1;
    [M1Map M1Trace M1RowShift M1ColShift M1OriShift] = ...
        CcomputeMAX1( nGaborOri, S1Map, locationShift, oriShift, sizeGaborPerScale, subsampleM1 );

    % compute S2 map
    S2Map = CcomputeSUM2( nGaborOri, M1Map, TransformedTemplate, subsampleS2 );

    % compute M2 map
    locationPerturbationFraction = 0.2; subsampleM2 = 1;
    M2Map = cell(nTransform, nS2);
    M2LocationTrace = cell(nTransform, nS2);
    M2TransformTrace = cell(nTransform, nS2);
    for iS2 = 1:nS2
        sizeTemplate = zeros(nTransform,1);
        for iT = 1:nTransform
            sizeTemplate(iT) = sqrt( ...
                size(TransformedTemplate{iT,iS2}.sym,1)...
                * size(TransformedTemplate{iT,iS2}.sym,2) ) / subsampleS2;
        end
        [M2Map(:,iS2) M2LocationTrace(:,iS2) M2TransformTrace(:,iS2) storedM2LocationShift] = ...
              CcomputeMAX2( templateAffMat, S2Map((iS2-1)*nTransform + (1:nTransform)), locationPerturbationFraction, ... 
                 int32(sizeTemplate), subsampleM2 );
        storedSizeS2Template{iS2} = sizeTemplate;
    end
    disp('done with computing M2');

    %% detection
    subsampleS3 = 1;
    %
    % Compute SUM 3 map :
    %
    S3Map = CcomputeSUM3_loglinear( M2Map, S3Template, subsampleS3, nTransform );
    selectedTransform = S3Template{1}.selectedTransform;
    selectedRow = S3Template{1}.selectedRow;
    selectedCol = S3Template{1}.selectedCol;
    selectedS2Ind = S3Template{1}.selectedInd;
    selectedLambda = S3Template{1}.selectedLambda;
    selectedLogZ = S3Template{1}.selectedLogZ;
    nF = length(selectedTransform);
    %
    %
    %
    storedS3Map = S3Map;
    minval = min(S3Map{1}(:));

    maxval = 10;
    count = 0;
    maxDetection = 1; % maximum number of detections, currently we only detect the maximum one 
   
    
    while maxval > -10
        count = count + 1;
        if count > maxDetection
            break;
        end

        % find the best position
        [maxval ind] = max(S3Map{1}(:));
        col = ceil(ind/size(S3Map{1},1));
        row = ind - (col-1) * size(S3Map{1},1);

        overlay = single(im);
        mask = zeros(size(im));
        for iF = 1:nF
            iT = selectedTransform(iF)+1;
            iS2 = selectedS2Ind(iF) + 1;

            part_row = row + selectedRow(iF);
            part_col = col + selectedCol(iF);
            if part_row <= 0 || part_col <= 0 || part_row > size(S3Map{1},1) || part_col > size(S3Map{1},2)
                continue;
            end

            % trace the deformation for each part
            part_loc_trace = M2LocationTrace{iT,iS2}(part_row,part_col) + 1;
            part_trans_trace = M2TransformTrace{iT,iS2}(part_row,part_col) + 1; % in matlab, the index start from 1 
            S2Size = storedSizeS2Template{iS2}(part_trans_trace); % size of the S2 template
            part_row = part_row * subsampleS2 + floor( .5 + storedM2LocationShift(part_loc_trace,1) * S2Size ) * subsampleS2;
            part_col = part_col * subsampleS2 + floor( .5 + storedM2LocationShift(part_loc_trace,2) * S2Size ) * subsampleS2;

            S2T = TransformedTemplate{part_trans_trace,iS2};

            for iGabor = 1:length(S2T.selectedRow)
                gabor_row = part_row + S2T.selectedRow(iGabor);
                gabor_col = part_col + S2T.selectedCol(iGabor);
                gabor_ori = S2T.selectedOri(iGabor) + 1;
                gabor_scale = S2T.selectedScale(iGabor) + 1;
                if gabor_row <= 0 || gabor_col <= 0 || gabor_row > size(im,1) || gabor_col > size(im,2)
                    continue;
                end
                gabor_trace = M1Trace{(gabor_scale-1)*nGaborOri+gabor_ori}(gabor_row,gabor_col) + 1;
                gabor_dr = single(M1RowShift{gabor_scale}(gabor_trace,gabor_ori));
                gabor_dc = single(M1ColShift{gabor_scale}(gabor_trace,gabor_ori));
                gabor_do = single(M1OriShift{gabor_scale}(gabor_trace,gabor_ori));
                gabor_row = gabor_row + gabor_dr;
                gabor_col = gabor_col + gabor_dc;
                gabor_ori = gabor_do + 1; % note: gabor_do is already the transformed orientation
                while gabor_ori > nGaborOri
                    gabor_ori = gabor_ori - nGaborOri;
                end
                while gabor_ori <= 0
                    gabor_ori = gabor_ori + nGaborOri;
                end

                % strength of this Gabor symbol (between 0 and 1)
                S1score = S1Map{(gabor_scale-1)*nGaborOri+gabor_ori}(gabor_row,gabor_col);
                S1score = S1score - 3; % These two parameters are just for display purpose
                priorS1 = 0.5;
                strength = priorS1 * exp(S1score) / ( (1-priorS1) + priorS1 * exp(S1score) );
                strength = max(0.8,strength);
                
                % Gabor symbol
                patch = strength * GaborSymbol{(gabor_scale-1)*nGaborOri+gabor_ori};
                
                % below: the bounding box for Gabor inside the image
                top2 = gabor_row - floor(size(patch,1)/2);
                left2 = gabor_col - floor(size(patch,2)/2);
                for c = 1:size(patch,2)
                    for r = 1:size(patch,1)
                        c2 = left2 + c - 1;
                        r2 = top2 + r - 1;
                        if r2 <= 0 || r2 > size(mask,1) || c2 <= 0 || c2 > size(mask,2)
                            continue;
                        end
                        val = patch(r,c);
                        if val > mask(r2,c2)
                            mask(r2,c2) = val;
                        end
                    end
                end
            end

        end

        h = figure;
        subplot(2,3,1);imshow(im+200*mask,[]);
        subplot(2,3,2);imshow(S3Map{1},[]);
        subplot(2,3,4);imshow(M2Map{selectedTransform(1)+1,selectedS2Ind(1)+1},[]);
        subplot(2,3,5);imshow(M2Map{selectedTransform(2)+1,selectedS2Ind(2)+1},[]);
        subplot(2,3,6);imshow(M2Map{selectedTransform(3)+1,selectedS2Ind(3)+1},[]);
        mkdir(sprintf('result/%s',category));
        saveas(h,sprintf('result/%s/detection%d_%s',category,count,Iname{iImage}));
        save(sprintf('result/%s/detectionResult%d_%s.mat',category,count,Iname{iImage}),...
            'im','mask','S3Map','M2Map', 'storedSizeS2Template');
        close;

        % inhibit the local region for the next detection
        for r = row - floor(S3size/2/subsampleS2) : row + floor(S3size/2/subsampleS2)
            for c = col - floor(S3size/2/subsampleS2) : col + floor(S3size/2/subsampleS2)
                if r <= 0 || c <= 0 || r > size(S3Map{1},1) || c > size(S3Map{1},2)
                    continue;
                end
                S3Map{1}(r,c) = minval;
            end
        end
    end
end
