% GenerteHtml - Generates html page for experimental results
%

addpath common

Iname = {'pelican 2_big.jpg','pelican3.jpg',...
    'pelican4.jpg','pelican5.jpg','pelican6.jpg',...
    'pelican7.jpg','pelican8.jpg','pelican9.jpg',...
    'pelican10.jpg'};
Iname_pelican = Iname;
imScale = [0.25 0.4 0.4 0.4 ...
    0.22 0.36 0.5 0.4 0.3];

category = 'pelican';
overlay_pelican = cell(length(imScale),1);
j_detection = 1;
for i = 1:length(imScale)
    load(sprintf('result/%s/detectionResult%d_%s.mat',category,j_detection,Iname{i}),...
                'im','mask','S3Map','M2Map');
    overlay_pelican{i} = im + 400 * mask;
end

Iname = {'l1.jpg','l2.jpg','l3.jpg','l4.jpg','l5.jpg',...
    'l6.jpg','l7.jpg','l8.jpg','l9.jpg','l10.jpg',...
    'l11.jpg','l12.jpg','l13.jpg','l14.jpg','l15.jpg',...
    'l16.jpg','l17.jpg','l18.jpg','l19.jpg','l20.jpg',...
    'l21.jpg', 'l22.jpg','l23.jpg','l24.jpg','l25.jpg',...
    'l26.jpg','l27.jpg','l28.jpg','l29.jpg','l30.jpg',...
    'l4_dup.jpg', 'l12_dup.jpg', 'l12_dup2.jpg'};
Iname_egret = Iname;
imScale = [1.2 1.2 1 2 1,...
    1 1.2 0.9 0.9 1,...
    0.95 1 1.1 1 1,...
    1.1 0.8 1.2 1.2 0.9,...
    .55 .3,.3 .25 .3 ...
    .3 .4,.3 .35 .3,...
    1.15 1.1 1.2];
category = 'egret';
overlay_egret = cell(length(imScale),1);
j_detection = 1;
for i = 1:length(imScale)
    load(sprintf('result/%s/detectionResult%d_%s.mat',category,j_detection,Iname{i}),...
                'im','mask','S3Map','M2Map');
    overlay_egret{i} = im + 400 * mask;
end

%%
selected_egret = 1:33;
selected_pelican = 1:9;

overlay = [overlay_egret(selected_egret);overlay_pelican(selected_pelican)];
Iname = [Iname_egret(selected_egret) Iname_pelican(selected_pelican)];

%% write the html code for reproducibility page

% delete the previous version
html_dir = 'document/html';
if ~exist(html_dir,'dir')
    mkdir(html_dir);
else
%     rmdir(html_dir,'s');
%     mkdir(html_dir);
end
html_path = sprintf('%s/exp.html',html_dir);
html = fopen(html_path,'w');

html_img_dir = 'document/html/img/';
if ~exist(html_img_dir,'dir')
    mkdir(html_img_dir);
end

tmp = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">\n';
tmp = [tmp '<html xmlns="http://www.w3.org/1999/xhtml">\n'];
tmp = [tmp '<head profile="http://gmpg.org/xfn/11">\n'];
tmp = [tmp '<title>Active Ellipsoid</title>\n'];
tmp = [tmp '</head>\n'];
fprintf(html,tmp);
fprintf(html, '<body>\n');
fprintf(html, '<h1> Active Ellipsoid Experiment </h1>');
fprintf(html, '<p> ');
fprintf(html, [' A two-level active basis model is contructed and used to detect egrets in images, with moderate deformation, ' ...
    'and location/scale uncertainty.' ...
    ]);
fprintf(html, ' </p> <br/>');

for k = 1:length(overlay)
    if k <= length(selected_egret)
        im = imread(sprintf('image/egret/%s',Iname{k}));
    else
        im = imread(sprintf('image/pelican/%s',Iname{k}));
    end
    
    for c = 1:3
        frames{2*(k-1)+1,c} = im(:,:,c); % source image
        frames{2*(k-1)+2,c} = overlay{k}; % overlay
    end
    
    for c = 1:3
        channel{c} = displayImages(frames(2*(k-1)+(1:2),c),2,size(im,1),size(im,2),...
            true,0,0);
    end
    
    towrite = zeros(size(channel{1}),'uint8');
    for c = 1:3
        towrite(:,:,c) = channel{c};
    end
    
    % eps figure
    figure;
    imshow(towrite);
    saveas(gcf,sprintf('document/html/img/%s.eps',Iname{k}),'epsc');
    close;
    % jpg figure
    imwrite(uint8(towrite),sprintf('document/html/img/%s',Iname{k}));
end

% move the shape script template to the image folder
src = 'result/egret/egret_template.eps';
dst = 'document/html/img/egret_template.eps';
copyfile(src,dst);
src = 'result/egret/template.png';
dst = 'document/html/img/egret_template.png';
copyfile(src,dst);


fprintf(html, '<hr /> \n');
fprintf(html, sprintf('<h3>%s</h3>','egret and pelican examples'));
tmp = sprintf('<p>The shape script is a two level active basis template: </p>\n');
tmp = [tmp '<img src=img/egret_template.png width=400 alt="" />'];
tmp = [tmp '<a href=img/egret_template.eps>eps</a>'];
fprintf(html, tmp);

tmp = sprintf('<p>The following figure illustrates the detection result using the above shape script. ');
tmp = [tmp '</p>\n'];
fprintf(html, tmp);

for k = 1:length(overlay)
    % output the figure to html
    new_img_name = sprintf('%s_activebasis.png',category);
    fprintf(html, sprintf('<img src="img/%s" alt="" height="100" /> \n',Iname{k}) );
    fprintf(html, sprintf('<a href=img/%s.eps>eps</a>\n',Iname{k}) );
end

fprintf(html, '</body> </html> \n');
fclose(html);

