% GenerateLatex - Generate a simple latex file together with a folder of eps figures
%   with detection results.

addpath common

Iname = {'pelican 2_big.jpg','pelican3.jpg',...
    'pelican4.jpg','pelican5.jpg','pelican6.jpg',...
    'pelican7.jpg','pelican8.jpg','pelican9.jpg',...
    'pelican10.jpg'};
Iname_pelican = Iname;
imScale = [0.25 0.4 0.4 0.4 ...
    0.22 0.36 0.5 0.4 0.3];

category = 'pelican';
overlay_pelican = cell(length(imScale),1);
j_detection = 1;
for i = 1:length(imScale)
    load(sprintf('result/%s/detectionResult%d_%s.mat',category,j_detection,Iname{i}),...
                'im','mask','S3Map','M2Map');
    overlay_pelican{i} = im + 400 * mask;
end

Iname = {'l1.jpg','l2.jpg','l3.jpg','l4.jpg','l5.jpg',...
    'l6.jpg','l7.jpg','l8.jpg','l9.jpg','l10.jpg',...
    'l11.jpg','l12.jpg','l13.jpg','l14.jpg','l15.jpg',...
    'l16.jpg','l17.jpg','l18.jpg','l19.jpg','l20.jpg',...
    'l21.jpg', 'l22.jpg','l23.jpg','l24.jpg','l25.jpg',...
    'l26.jpg','l27.jpg','l28.jpg','l29.jpg','l30.jpg',...
    'l4_dup.jpg', 'l12_dup.jpg', 'l12_dup2.jpg'};
Iname_egret = Iname;
imScale = [1.2 1.2 1 2 1,...
    1 1.2 0.9 0.9 1,...
    0.95 1 1.1 1 1,...
    1.1 0.8 1.2 1.2 0.9,...
    .55 .3,.3 .25 .3 ...
    .3 .4,.3 .35 .3,...
    1.15 1.1 1.2];
category = 'egret';
overlay_egret = cell(length(imScale),1);
j_detection = 1;
for i = 1:length(imScale)
    load(sprintf('result/%s/detectionResult%d_%s.mat',category,j_detection,Iname{i}),...
                'im','mask','S3Map','M2Map');
    overlay_egret{i} = im + 400 * mask;
end

%%
% selected_egret = setdiff(1:33,[6 11 12 13 16 21 23 24 25 27 28 29]);
selected_egret = setdiff(1:33,[6 7 9 11 13 16 19 20 21 23 24 25 27 28 29 30]);
selected_egret = 1:33;
selected_pelican = [7:9];
selected_pelican = 1:9;

overlay = [overlay_egret(selected_egret);overlay_pelican(selected_pelican)];
Iname = [Iname_egret(selected_egret) Iname_pelican(selected_pelican)];

frames = cell(2*length(overlay),3);
% now interleve original image with overlay
for k = 1:length(overlay)
    if k <= length(selected_egret)
        im = imread(sprintf('image/egret/%s',Iname{k}));
        currentCategory = 'egret';
    else
        im = imread(sprintf('image/pelican/%s',Iname{k}));
        currentCategory = 'pelican';
    end
    
    for c = 1:3
        frames{2*(k-1)+1,c} = im(:,:,c); % source image
        frames{2*(k-1)+2,c} = overlay{k}; % overlay
    end
    
    for c = 1:3
%         channel{c} = [im(:,:,c) , overlay{k}];
        channel{c} = displayImages(frames(2*(k-1)+(1:2),c),2,size(im,1),size(im,2),...
            true,0,0);
    end
    
    towrite = zeros(size(channel{1}),'uint8');
    for c = 1:3
        towrite(:,:,c) = channel{c};
    end
    
    figure;
    imshow(towrite);
    saveas(gcf,sprintf('result/%s/%s.eps',currentCategory,Iname{k}),'epsc');
    close;
end

%% also generate the corresponding latex
tex_dir = 'document/latex';
if ~exist(tex_dir,'dir')
    mkdir(tex_dir);
else
%     rmdir(tex_dir,'s');
    mkdir(tex_dir);
end
tex_path = sprintf('%s/exp.tex',tex_dir);
tex = fopen(tex_path,'w');

tex_img_dir = 'document/latex/figure/';
if ~exist(tex_img_dir,'dir')
    mkdir(tex_img_dir);
end

tmp = ['\\documentclass[11pt]{article}\n' ...
    '\\usepackage{epsfig}\n'];
fprintf(tex,tmp);
fprintf(tex,'\\title{Report}\n');
fprintf(tex,'\\begin{document}\n');
fprintf(tex,'\\maketitle\n');

tmp = [''];
fprintf(tex,tmp);

% move eps figures to "<root>/document/latex/figure/"
fprintf(tex, '\n \\begin{figure}[htp] \n');
for k = 1:length(overlay)
    if k <= length(selected_egret)
        im = imread(sprintf('image/egret/%s',Iname{k}));
        currentCategory = 'egret';
    else
        im = imread(sprintf('image/pelican/%s',Iname{k}));
        currentCategory = 'pelican';
    end

    % copy eps figures to "<root>/document/tex/img/"
    src = sprintf('result/%s/%s.eps',currentCategory,Iname{k});
    dst = sprintf('document/latex/figure/%s_%s.eps', currentCategory,Iname{k});
    copyfile(src,dst);
    
    % output the figure and caption to tex : active basis
    fprintf(tex, sprintf('\\\\psfig{figure=figure/%s_%s.eps, height=300pt }\n', currentCategory, Iname{k}));
    
end
tmp = '\\caption{}';
fprintf(tex, tmp);
fprintf(tex, '\\label{fig:pelican}\n');
fprintf(tex, '\\end{figure} \n');

fprintf(tex, '\\end{document} \n');
fclose(tex);