% StartFromHere - The entry point of this project.
%
%   StartFromHere tests retrieval image visual objects from a shape script template.
%
%   The shape script template is composed of 4 shape motifs (basic shapes, 
%   e.g. ellipsoid, angle, parallel bars), which are learned from a single
%   clean example image. The search for the shape script in the testing 
%   image is performed by a dynamic programming approach, which we call
%   recursive SUM/MAX procedure. It results in SUM1/MAX1, SUM2/MAX2 and 
%   SUM3 maps. And a best location is found in SUM3 map that has the
%   maximum of matching score.
%

%% compilation
cd common
mex Chistogram.c
mex CcomputeMAX1.c
mex CgetMAX1track.c
mex CcomputeSUM2.c
mex CcomputeMAX2.c
mex CcomputeSUM3_loglinear.c
mex CsharedSketch.c
mex Csigmoid.c
mex ClocalNormalizeSingle.c
cd ..

%% set parameters
setTemplateParameters;

%% precompute S1 maps on natural images
if ~exist('feature/neg/neg02.jpg.mat','file')
    cd common
    dumpNeg;
    cd ..
end

%% obtain natural statistics
if ~exist('result/natural_stat.mat','file')
    cd common
    getNaturalStat_multiscale;
    cd ..
end


%% learn shape motifs
cd common
LearnS2Template;
generateTransformedS2Templates;
cd ..

%% shape script
ConstructEgretShapeScript;

%% Detect 
FindEgret;
FindPelican;

%% generate figures
% GenerateEgretFigure;    % (obsolete) generate a single eps figure with detection results
% GenerateLatex;      % (obsolete) generate a simle latex file together with a folder of eps detection results

%% generate project page
GenerateHtml;       % generate a simle latex file together with a folder of eps and jpg detection results

%% Compare the distribution of S3 scores on positive/negative examples
DisplayS3ScoreHistogram;
