% LearnEllipsoidTemplate - Learns an active basis template for Ellipsoid

clear all; close all; 
disp('===============> Learning the active basis model from positive training images');

load ../template_params

%% generate the training image
numImage = 1; % number of training images
I = cell(numImage, 1);
sizex = 63; sizey = 63;
center = 32; a = 22; b = 10;
I0 = single(zeros(sizex, sizey)); 
for x = 1:sizex
        for y = 1:sizey
            if (((x-center)/a)^2+((y-center)/b)^2<1.)
                I0(x, y) = single(1.); 
            end
        end
end
I{1} = I0+0.;

allSizex = zeros(1, numImage, 'single')+sizex; 
allSizey = zeros(1, numImage, 'single')+sizey;
%% learning a template


% use the 3rd scale
s = 4;
storeName = ['../result/naturalStat' num2str(s-1)];
load(storeName);
numElement = 14;
halfFilterSize = ( size(GaborFilter{(s-1)*nGaborOri+1},1) - 1 ) / 2;
disp(['start filtering training images at Gabor length ' num2str(2*halfFilterSize+1)]); tic

selectedOrient = zeros(numElement, 1, 'single');  % orientation and location of selected Gabors
selectedx = zeros(numElement, 1, 'single');
selectedy = zeros(numElement, 1, 'single');
selectedlambda = zeros(numElement, 1, 'single'); % weighting parameter for scoring template matching
selectedLogZ = zeros(numElement, 1, 'single'); % normalizing constant
symbolicTemplate = zeros(sizex, sizey, 'single'); % template of active basis 


%% Compute SUM1 maps by Gabor filtering
SUM1map = ApplyfilterfftsameLocalNormPerOri(I, GaborFilter((s-1)*nGaborOri+(1:nGaborOri))); % SUM1 maps by Gabor filtering
disp(['filtering time: ' num2str(toc) ' seconds']);

%% Compute MAX1 maps and record the track maps
locationShiftLimit = 0; % shift in normal direction = locationShiftLimit*subsample pixels
orientShiftLimit = 0; % shift in orientation
Csigmoid(numImage, allSizex, allSizey, nGaborOri, GaborSaturation, SUM1map);
MAX1map = cell(numImage, nGaborOri);
subsampleM1 = 1;  % to be safe, please refrain from setting it to be larger than 1 
trackMap = cell(numImage, nGaborOri);
for img = 1:numImage
    for orient = 1:nGaborOri
        MAX1map{img, orient} = single(zeros(floor(sizex/subsampleM1), floor(sizey/subsampleM1))); 
        trackMap{img, orient} = single(zeros(floor(sizex/subsampleM1), floor(sizey/subsampleM1))); 
    end
end
disp(['start maxing training images at Gabor length ' num2str(2*halfFilterSize+1)]); tic
CgetMAX1track(numImage, allSizex, allSizey, nGaborOri, locationShiftLimit, orientShiftLimit, subsampleM1, ...
              SUM1map, MAX1map, trackMap);
disp(['maxing time: ' num2str(toc) ' seconds']);


%% Learning by shared sketch algorithm
learningSampleSize = 1;
deformedTemplate = cell(1, learningSampleSize); % templates for training images
for img = 1 : learningSampleSize
    deformedTemplate{img} = single(zeros(sizex, sizey));  
end
SUM2score = zeros(learningSampleSize, 1); % template matching scores for training images 
disp(['start mex-C learning of template at Gabor length ' num2str(2*halfFilterSize+1)]); tic
CsharedSketch(nGaborOri, locationShiftLimit, orientShiftLimit, GaborSaturation, subsampleM1, ... % about active basis  
       numElement, learningSampleSize, sizex, sizey, ...
       SUM1map, MAX1map, trackMap, ... % about training images 
       halfFilterSize, CorrGabor((s-1)*nGaborOri + (1:nGaborOri), (s-1)*nGaborOri + (1:nGaborOri)),...
       GaborSymbol( (s-1)*nGaborOri + (1:nGaborOri) ), ... % about filters
       numStoredPoint, (storedLambda), (storedExpectation), (storedLogZ), ... % about exponential model 
       selectedOrient, selectedx, selectedy, selectedlambda, selectedLogZ, SUM2score, ... % learned parameters
       symbolicTemplate, deformedTemplate); % learned templates

selectedScale = ones(numElement,1)*(s-1); % the scale index starts from 0
   
    %% Display and save results
%         OutLearningResult;
%         Outepsgif(-symbolicTemplate,
%         ['output/learnedTemplateUsingGaborScale' num2str(s)]); % output
%         learned template

disp(['mex-C learning time: ' num2str(toc) ' seconds']);
templateName = ['../result/learnedEllipsoidTemplate' num2str(s-1)];
save(templateName, 'numElement', 'selectedOrient', 'selectedx', 'selectedy', 'selectedScale', 'selectedlambda', 'selectedLogZ', ...
                   'sizex', 'sizey', 'symbolicTemplate', 'subsampleM1');

