% generateTransformedS2Templates - Affine transforms of the S2
%   templates (shape motifs).
clear
load ../template_params nGaborOri nGaborScale GaborSymbol
templateName = ['../result/learnedS2Template'];
load(templateName);
nS2T = length(S2Template);

%% enumerate the transformations
count = 0;
for templateScaleInd = -2:1 % large scale change
    for rotation = 0:2:(nGaborOri-1) % 0 ~ pi is enough for ellisoid
        for rowScale = 2.^[0, 1/4] % small scale change and aspect ratio change
            for colScale = 2.^[1/4, 0]
                count = count + 1;
                templateTransform{count} = [templateScaleInd rowScale colScale rotation];
            end
        end
    end
end
nTransform = count;

%% the actual transformation computation
TransformedTemplate = cell(nTransform,nS2T);

maxSquareSize = 0;
for iS2 = 1:nS2T
    selectedx = S2Template{iS2}.selectedx;
    selectedy = S2Template{iS2}.selectedy;
    selectedOrient = S2Template{iS2}.selectedOrient;
    selectedScale = S2Template{iS2}.selectedScale;
    numElement = S2Template{iS2}.numElement;
    for iT = 1:nTransform
        templateScaleInd = templateTransform{iT}(1);
        rowScale = templateTransform{iT}(2);
        colScale = templateTransform{iT}(3);
        rotation = templateTransform{iT}(4);
        tmpSelectedRow = zeros(numElement,1,'single');
        tmpSelectedCol = zeros(numElement,1,'single');
        tmpSelectedOri = zeros(numElement,1,'single');
        tmpSelectedScale = zeros(numElement,1,'single');
        for iF = 1:numElement
            inRow = selectedx(iF);
            inCol = selectedy(iF);
            inO = selectedOrient(iF); % start from 0
            inS = selectedScale(iF);
            [tmpSelectedRow(iF), tmpSelectedCol(iF), tmpSelectedOri(iF) tmpSelectedScale(iF)] = ...
                templateAffineTransform(templateScaleInd,rowScale,colScale,rotation,...
                inRow,inCol,inO,inS,nGaborOri);
        end
        TransformedTemplate{iT,iS2}.selectedRow = tmpSelectedRow;
        TransformedTemplate{iT,iS2}.selectedCol = tmpSelectedCol;
        TransformedTemplate{iT,iS2}.selectedOri = tmpSelectedOri;
        TransformedTemplate{iT,iS2}.selectedScale = tmpSelectedScale;
        TransformedTemplate{iT,iS2}.selectedLambda = S2Template{iS2}.selectedlambda;
        newSelectedLogZ = zeros(1,numElement);
        TransformedTemplate{iT,iS2}.selectedLogZ = S2Template{iS2}.selectedLogZ;
        TransformedTemplate{iT,iS2}.subsampleM1 = S2Template{iS2}.subsampleM1;
        TransformedTemplate{iT,iS2}.sym = displayTemplate( ...
            tmpSelectedRow,tmpSelectedCol,tmpSelectedOri, tmpSelectedScale, ones(numElement,1),...
            GaborSymbol,nGaborOri,S2Template{iS2}.subsampleM1);
        if size(TransformedTemplate{iT,iS2}.sym,1) > maxSquareSize
            maxSquareSize = size(TransformedTemplate{iT,iS2}.sym,1);
        end
    end
end

%% display the transformed templates
for iS2 = 1:nS2T
    tmpTT = cell(1,nTransform);
    for iT = 1:nTransform
        % inscribe into a fixed-size square
        tmpTT{iT} = zeros(maxSquareSize,maxSquareSize);
        squareSize = size(TransformedTemplate{iT,iS2}.sym,1);
        margin = floor((maxSquareSize - squareSize)/2);
        tmpTT{iT}(margin+(1:squareSize),margin+(1:squareSize)) = -single(TransformedTemplate{iT,iS2}.sym);
    end
    im = displayImages(tmpTT,4*2,50,50);
    imwrite(im,sprintf('../result/transformedTemplates_%d.png',iS2));
    figure; imshow(im,[]);
    saveas(gcf,sprintf('../result/transformedTemplates_%d.eps',iS2), 'epsc');
    close;
end

%% compute pairwise correlation
load ../template_params CorrGabor
save ../result/transformedS2Template.mat TransformedTemplate templateTransform



