% generateTransformedTemplates - Affine transforms of the ellipsoid
%   template.

EllipseGaborScale = 4;
load ../template_params nGaborOri nGaborScale GaborSymbol
templateName = ['../result/learnedEllipsoidTemplate' num2str(EllipseGaborScale-1)];
load(templateName);
height = sizex; width = sizey;

%% enumerate the transformations
count = 0;
for templateScaleInd = -2:1 % large scale change
    for rotation = 0:2:(nGaborOri-1) % 0 ~ pi is enough for ellisoid
        for rowScale = 2.^[0, 1/4] % small scale change and aspect ratio change
            for colScale = 2.^[1/4, 0]
                count = count + 1;
                templateTransform{count} = [templateScaleInd rowScale colScale rotation];
            end
        end
    end
end
nTransformedTemplates = count;

%% the actual transformation
TransformedTemplate = cell(nTransformedTemplates,1);
tmpTT = cell(nTransformedTemplates,1);
maxSquareSize = 0;
for iT = 1:nTransformedTemplates
    templateScaleInd = templateTransform{iT}(1);
    rowScale = templateTransform{iT}(2);
    colScale = templateTransform{iT}(3);
    rotation = templateTransform{iT}(4);
    tmpSelectedRow = zeros(numElement,1,'single');
    tmpSelectedCol = zeros(numElement,1,'single');
    tmpSelectedOri = zeros(numElement,1,'single');
    tmpSelectedScale = zeros(numElement,1,'single');
    for iF = 1:numElement
        inRow = selectedx(iF);
        inCol = selectedy(iF);
        inO = selectedOrient(iF); % start from 0
        inS = EllipseGaborScale;
        [tmpSelectedRow(iF), tmpSelectedCol(iF), tmpSelectedOri(iF) tmpSelectedScale(iF)] = ...
            templateAffineTransform(templateScaleInd,rowScale,colScale,rotation,...
            inRow,inCol,inO,inS,nGaborOri);
    end
    TransformedTemplate{iT}.selectedRow = tmpSelectedRow;
    TransformedTemplate{iT}.selectedCol = tmpSelectedCol;
    TransformedTemplate{iT}.selectedOri = tmpSelectedOri;
    TransformedTemplate{iT}.selectedScale = tmpSelectedScale;
    TransformedTemplate{iT}.selectedLambda = selectedlambda;
    newSelectedLogZ = zeros(1,numElement);
    TransformedTemplate{iT}.selectedLogZ = selectedLogZ;
    TransformedTemplate{iT}.subsampleM1 = subsampleM1;
    TransformedTemplate{iT}.sym = displayTemplate( ...
        tmpSelectedRow,tmpSelectedCol,tmpSelectedOri, tmpSelectedScale, ones(numElement,1),...
        GaborSymbol,nGaborOri,subsampleM1);
    if size(TransformedTemplate{iT}.sym,1) > maxSquareSize
        maxSquareSize = size(TransformedTemplate{iT}.sym,1);
    end
end

%% display the transformed templates
for iT = 1:nTransformedTemplates
    % inscribe into a fixed-size square
    tmpTT{iT} = zeros(maxSquareSize,maxSquareSize);
    squareSize = size(TransformedTemplate{iT}.sym,1);
    margin = floor((maxSquareSize - squareSize)/2);
    tmpTT{iT}(margin+(1:squareSize),margin+(1:squareSize)) = -single(TransformedTemplate{iT}.sym);
end

im = displayImages(tmpTT,4*2,50,50);
imwrite(im,'../result/transformedTemplates.png');


%% compute pairwise correlation
load ../template_params CorrGabor
subsampleS2 = 5; inhibitionThresS2 = 0;
save ../result/transformedEllipsoids.mat subsampleS2 inhibitionThresS2 TransformedTemplate EllipseGaborScale subsampleM1
% CorrS2 = CcomputeCorrS2( nGaborOri, CorrGabor, TransformedTemplate, subsampleS2, inhibitionThresS2);
% save ../result/transformedEllipsoids.mat CorrS2 subsampleS2 inhibitionThresS2 TransformedTemplate EllipseGaborScale subsampleM1

%% Following (obsolete): the matlab code for computing the correlation maps between S2 templates. 
%
% for iT = 1:nTransformedTemplates
%     for jT = 1:nTransformedTemplates % floating template
%         % to do: write a mex-C module to deal with computation of I
%         
%         I = zeros( 2 * floor( size(TransformedTemplate{iT}.sym,1)/2 ) + 2 * floor( size(TransformedTemplate{jT}.sym,1)/2 ) + 1 );
%         halfSize = floor(size(I,1)/2);
%         for row = 1:size(I,1)
%             for col = 1:size(I,2)
%                 
%                 count = 0;
%                 
%                 for iF = 1:length(TransformedTemplate{iT}.selectedRow)
%                     % check if there is any overlapping
%                     is_overlap = 0;
%                     iOri = TransformedTemplate{iT}.selectedOri(iF);
%                     iS = TransformedTemplate{iT}.selectedScale(iF);
%                     iRow = TransformedTemplate{iT}.selectedRow(iF);
%                     iCol = TransformedTemplate{iT}.selectedCol(iF);
%                     if iS < 1 || iS > nGaborScale
%                         continue;
%                     end
%                     for jF = 1:length(TransformedTemplate{jT}.selectedRow)
%                         jOri = TransformedTemplate{jT}.selectedOri(jF);
%                         jS = TransformedTemplate{jT}.selectedScale(jF);
%                         jRow = TransformedTemplate{jT}.selectedRow(jF);
%                         jCol = TransformedTemplate{jT}.selectedCol(jF);
%                         halfSizeCorrGabor = ( size( CorrGabor{(iS-1)*nGaborOri+iOri, (jS-1)*nGaborOri+jOri}, 1 ) - 1 ) / 2;
%                         if abs( (row + jRow) - (halfSize + iRow) ) >=...
%                                 halfSizeCorrGabor || abs( (col + jCol) - (halfSize + iCol) ) >=...
%                                 halfSizeCorrGabor % no overlap
%                             continue;
%                         end
%                         if CorrGabor{(iS-1)*nGaborOri+iOri, (jS-1)*nGaborOri+jOri}(...
%                                 (row + jRow) - (halfSize + iRow) + halfSizeCorrGabor, ...
%                                 (col + jCol) - (halfSize + iCol) + halfSizeCorrGabor ) < 1
%                             is_overlap = 1;
%                             break;
%                         end
%                     end
%                     if is_overlap > 0
%                         count = count + 1;
%                     end
%                 end
%                 
%                 if double(count) / double(length(TransformedTemplate{iT}.selectedRow)) < 0.2
%                     I(row,col) = 1; % 1 means no inhibition
%                 end
%             end
%         end
%         
%         CorrS2{iT,jT} = I;
%     end
% end



