function getNaturalStat_multiscale
% getNaturalStat_multiscale - Obtain natural statistics for multiscale Gabor responses 

load ../template_params.mat
negImageNames = dir('../image/neg/*.jpg');
negImageNames = negImageNames(1:min(5,end));
numImage = length(negImageNames);
mkdir('../result');

for s = 1 : nGaborScale
    scaleFilter = GaborScale(s);
    %% compute histogram of q()
    numBin = floor(GaborSaturation/binSize)+1;  % binnumbers
    histog = zeros(numBin, 1, 'single');  % store F
    r = (0:(numBin-1))*binSize;
    halfFilterSize = ( size(GaborFilter{nGaborOri*(s-1)+1},1)-1 ) / 2;
    disp(['start histogramming negative images at Gabor length ' num2str(2*halfFilterSize+1)]);
    
    allFilteredImage = cell(length(negImageNames), nGaborOri);
    sizex = zeros(numImage,1,'int32');
    sizey = zeros(numImage,1,'int32');
    for i = 1:numImage
        load(sprintf('../feature/neg/%s.mat',negImageNames(i).name),'S1Map');
        allFilteredImage(i,:) = S1Map(nGaborOri*(s-1)+(1:nGaborOri));
        sizex(i) = size(allFilteredImage{i,1},1);
        sizey(i) = size(allFilteredImage{i,1},2);
    end

    Chistogram(numImage, nGaborOri, allFilteredImage, halfFilterSize, sizex, sizey, binSize, numBin, histog, GaborSaturation);
    %% compute stored lambda, expectation, logZ

    storedExpectation = zeros(numStoredPoint, 1); 
    Z = zeros(numStoredPoint, 1); 
    for k=1:numStoredPoint
       lambda = (k-1.)*spacing; 
       p = exp(lambda*r).*(histog'); 
       Z(k) = sum(p*binSize); p = p/Z(k); 
       storedExpectation(k) = sum(r.*p*binSize);
    end
    storedLambda = (0:(numStoredPoint-1))*spacing; 
    storedLogZ = log(Z);

    %% store filter properties and exponential model
    storeName = ['../result/naturalStat' num2str(s-1) '.mat'];
    if SinglePrecision > 0
        storedLambda = single(storedLambda);
        storedExpectation = single(storedExpectation);
        storedLogZ = single(storedLogZ);
        r = single(r);
        histog = single(histog);
    end
    save(storeName, 'storedLambda', 'storedExpectation', 'storedLogZ', 'numBin', 'r', 'histog'); 
end

%% display the comparison
rMat = zeros(nGaborScale,length(r));
histogMat = zeros(nGaborScale,length(r));
for s = [1 4 7]
    load(sprintf('../result/naturalStat%d.mat',s-1),'r','histog');
    rMat(s,:) = r(:);
    histogMat(s,:) = histog(:);
end

figure; plot(rMat([1 4 7],:)',histogMat([1 4 7],:)');
legend({'1','4','7'});
title('comparison of background density'); xlabel('sigmoid(response)'); ylabel('density');
set(gca,'FontSize',16); % adjust the display format and save the plot
set(get(gca,'XLabel'),'FontSize',18);
set(get(gca,'YLabel'),'FontSize',18);
set(get(gca,'Title'),'FontSize',18);
set(gcf, 'PaperPositionMode', 'auto');
print('-dpng','../result/background_density_comparison_over_scale.png');
saveas(gcf, '../result/background_density_comparison_over_scale.eps', 'epsc');

lambdaMat = zeros(nGaborScale,length(storedLambda));
expMat = zeros(nGaborScale,length(storedLambda));
for s = [1 4 7]
    load(sprintf('../result/naturalStat%d.mat',s-1),'storedLambda','storedExpectation');
    lambdaMat(s,:) = storedLambda(:);
    expMat(s,:) = storedExpectation(:);
end

figure; plot(lambdaMat([1 4 7],:)',expMat([1 4 7],:)');
title('comparison of lambda-expectation relation'); xlabel('lambda'); ylabel('expected value');
legend({'1','4','7'},'Location','South');
set(gca,'FontSize',16); % adjust the display format and save the plot
set(get(gca,'XLabel'),'FontSize',18);
set(get(gca,'YLabel'),'FontSize',18);
set(get(gca,'Title'),'FontSize',18);
set(gcf, 'PaperPositionMode', 'auto');
print('-dpng','../result/lambda_exp_comparison_over_scale.png');
saveas(gcf, '../result/lambda_exp_comparison_over_scale.eps', 'epsc');
